library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity project is
  port (
    -- Clocks
    
    CLOCK_50 : in std_logic;                       -- 50 MHz
    
	 -- SRAM
    
    SRAM_DQ : inout std_logic_vector(15 downto 0); -- Data bus 16 Bits
    SRAM_ADDR : out std_logic_vector(17 downto 0); -- Address bus 18 Bits
    SRAM_UB_N,                                     -- High-byte Data Mask 
    SRAM_LB_N,                                     -- Low-byte Data Mask 
    SRAM_WE_N,                                     -- Write Enable
    SRAM_CE_N,                                     -- Chip Enable
    SRAM_OE_N : out std_logic;                     -- Output Enable
    
	 -- VGA output
    
    VGA_CLK,                                            -- Clock
    VGA_HS,                                             -- H_SYNC
    VGA_VS,                                             -- V_SYNC
    VGA_BLANK,                                          -- BLANK
    VGA_SYNC : out std_logic;                           -- SYNC
    VGA_R,                                              -- Red[9:0]
    VGA_G,                                              -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0);           -- Blue[9:0]
	 
	 -- PS/2 port

    PS2_DAT,                    -- Data
    PS2_CLK : inout std_logic     -- Clock
	 
    );  
end project;

architecture rtl of project is

signal counter : unsigned(15 downto 0);
signal reset_n : std_logic;

begin

process (CLOCK_50)
  begin
    if rising_edge(CLOCK_50) then
      if counter = x"ffff" then
        reset_n <= '1';
      else
        reset_n <= '0';
        counter <= counter + 1;
      end if;
    end if;
  end process;

  nios : entity work.nios_system port map (
    clk_0                         => CLOCK_50,
    reset_n                       => reset_n,
    SRAM_ADDR_from_the_sram       => SRAM_ADDR,
    SRAM_CE_N_from_the_sram       => SRAM_CE_N,
    SRAM_DQ_to_and_from_the_sram  => SRAM_DQ,
    SRAM_LB_N_from_the_sram       => SRAM_LB_N,
    SRAM_OE_N_from_the_sram       => SRAM_OE_N,
    SRAM_UB_N_from_the_sram       => SRAM_UB_N,
    SRAM_WE_N_from_the_sram       => SRAM_WE_N,
	 
	 VGA_CLK_from_the_vga_raster   => VGA_CLK,
    VGA_HS_from_the_vga_raster    => VGA_HS,
    VGA_VS_from_the_vga_raster    => VGA_VS,
    VGA_BLANK_from_the_vga_raster => VGA_BLANK,
    VGA_SYNC_from_the_vga_raster  => VGA_SYNC,
    VGA_R_from_the_vga_raster     => VGA_R,
    VGA_G_from_the_vga_raster     => VGA_G,
    VGA_B_from_the_vga_raster     => VGA_B,
	 
	 PS2_CLK_to_and_from_the_ALT_UP_PS2_0   => PS2_CLK,
	 PS2_DAT_to_and_from_the_ALT_UP_PS2_0   => PS2_DAT
    );

end rtl;
