-- Quartus II VHDL Template
-- Simple Dual-Port RAM with different read/write addresses but
-- single read/write clock

library ieee;
use ieee.std_logic_1164.all;

entity simple_dual_port_ram_single_clock_color is

	generic 
	(
		DATA_WIDTH : natural := 3;
		ADDR_WIDTH : natural := 6
	);

	port 
	(
		clk	: in std_logic;
		raddr	: in natural range 0 to 2**ADDR_WIDTH - 1;
		waddr	: in natural range 0 to 2**ADDR_WIDTH - 1;
		data	: in std_logic_vector((DATA_WIDTH-1) downto 0);
		we		: in std_logic := '1';
		q		: out std_logic_vector((DATA_WIDTH -1) downto 0)
	);

end simple_dual_port_ram_single_clock_color;

architecture rtl of simple_dual_port_ram_single_clock_color is

	-- Build a 2-D array type for the RAM
	subtype word_t is std_logic_vector((DATA_WIDTH-1) downto 0);
	type memory_t is array(2**ADDR_WIDTH-1 downto 0) of word_t;

	function init_ram
		return memory_t is 
		variable tmp : memory_t := (others => (others => '0'));
	begin 
		tmp(0)  := "011"; tmp(16) := "110"; tmp(32) := "101"; tmp(48) := "010";
		tmp(1)  := "011"; tmp(17) := "110"; tmp(33) := "101"; tmp(49) := "010";
		tmp(2)  := "011"; tmp(18) := "100"; tmp(34) := "101"; tmp(50) := "010";
		tmp(3)  := "011"; tmp(19) := "100"; tmp(35) := "101"; tmp(51) := "010";
		tmp(4)  := "011"; tmp(20) := "100"; tmp(36) := "001"; tmp(52) := "010";
		tmp(5)  := "011"; tmp(21) := "100"; tmp(37) := "001"; tmp(53) := "010";
		tmp(6)  := "011"; tmp(22) := "100"; tmp(38) := "001"; tmp(54) := "000";
		tmp(7)  := "011"; tmp(23) := "100"; tmp(39) := "001"; tmp(55) := "000";
		tmp(8)  := "011"; tmp(24) := "100"; tmp(40) := "001"; tmp(56) := "000";
		tmp(9)  := "110"; tmp(25) := "100"; tmp(41) := "001"; tmp(57) := "000";
		tmp(10) := "110"; tmp(26) := "100"; tmp(42) := "001"; tmp(58) := "000";
		tmp(11) := "110"; tmp(27) := "101"; tmp(43) := "001"; tmp(59) := "000";
		tmp(12) := "110"; tmp(28) := "101"; tmp(44) := "001"; tmp(60) := "000";
		tmp(13) := "110"; tmp(29) := "101"; tmp(45) := "010"; tmp(61) := "000";
		tmp(14) := "110"; tmp(30) := "101"; tmp(46) := "010"; tmp(62) := "000";
		tmp(15) := "110"; tmp(31) := "101"; tmp(47) := "010"; tmp(63) := "111";
		return tmp;
	end init_ram;	 

	-- Declare the RAM signal and specify a default value.	Quartus II
	-- will create a memory initialization file (.mif) based on the 
	-- default value.
	signal ram : memory_t := init_ram;

	-- Register to hold the address 
	signal addr_reg : natural range 0 to 2**ADDR_WIDTH-1;
	
	
begin

	process(clk)
	begin
	if(rising_edge(clk)) then 
		if(we = '1') then
			ram(waddr) <= data;
		end if;
 
		-- On a read during a write to the same address, the read will
		-- return the OLD data at the address
		q <= ram(raddr);
	end if;
	end process;

end rtl;
